<?php

namespace DBTech\Security\Admin\Controller;

use DBTech\Security\Helper;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

/**
 * Class CountryBlock
 * @package DBTech\Security\Admin\Controller
 */
class CountryBlock extends AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('dbtechSecurity');
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\PrintableException
	 */
	public function actionIndex(): \XF\Mvc\Reply\AbstractReply
	{
		$countries = Helper::repository(\DBTech\Security\Repository\Country::class)
			->findCountriesForList()
			->fetch()
		;

		if ($this->isPost())
		{
			/** @var \DBTech\Security\Entity\Country[]|\XF\Mvc\Entity\ArrayCollection $currentBlocked */
			$currentBlocked = $countries->filter(function (\DBTech\Security\Entity\Country $country): ?\DBTech\Security\Entity\Country
			{
				if ($country->blocked)
				{
					return $country;
				}

				return null;
			});

			$newBlocked = $this->filter('countries', 'array-str');
			foreach ($currentBlocked as $country)
			{
				if (!in_array($country->country_code, $newBlocked))
				{
					$country->blocked = false;
					$country->save();
				}
			}

			foreach ($newBlocked as $countryCode)
			{
				if (!$countries->offsetExists($countryCode))
				{
					continue;
				}

				/** @var \DBTech\Security\Entity\Country $country */
				$country = $countries->offsetGet($countryCode);

				if (!$country->blocked)
				{
					$country->blocked = true;
					$country->save();
				}
			}

			return $this->redirect($this->buildLink('dbtech-security/country-block'));
		}

		$viewParams = [
			'countries' => $countries
		];
		return $this->view(
			\DBTech\Security\Admin\View\CountryBlock\Index::class,
			'dbtech_security_country_block',
			$viewParams
		);
	}
}