<?php

namespace DBTech\Security\Admin\Controller;

use DBTech\Security\Helper;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

/**
 * Class Log
 * @package DBTech\Security\Admin\Controller
 */
class Log extends AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('dbtechSecurity');
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionIndex(): \XF\Mvc\Reply\AbstractReply
	{
		return $this->view(
			\DBTech\Security\Admin\View\Log::class,
			'dbtech_security_logs'
		);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \Exception
	 */
	public function actionAdminStrikes(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		if ($params->admin_strike_id)
		{
			$entry = $this->assertAdminStrikeExists($params->admin_strike_id, [
				'User'
			], 'requested_log_entry_not_found');

			$viewParams = [
				'entry' => $entry,
			];
			return $this->view(
				\DBTech\Security\Admin\View\Log\AdminStrikes\View::class,
				'dbtech_security_log_admin_strikes_view',
				$viewParams
			);
		}

		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\AdminStrike $searcher */
		$searcher = $this->searcher('DBTech\Security:AdminStrike', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\AdminStrikes\Listing::class,
			'dbtech_security_log_admin_strikes_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionAdminStrikesSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getAdminStrikeSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\AdminStrikes\Search::class,
			'dbtech_security_log_admin_strikes_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 */
	protected function getAdminStrikeSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\AdminStrike $searcher */
		$searcher = $this->searcher('DBTech\Security:AdminStrike');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param int|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\AdminStrike
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertAdminStrikeExists(?int $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\AdminStrike
	{
		return $this->assertRecordExists('DBTech\Security:AdminStrike', $id, $with, $phraseKey);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \Exception
	 */
	public function actionLoginStrikes(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		if ($params->login_strike_id)
		{
			$entry = $this->assertLoginStrikeExists($params->login_strike_id, [
			], 'requested_log_entry_not_found');

			$viewParams = [
				'entry' => $entry,
			];
			return $this->view(
				\DBTech\Security\Admin\View\Log\LoginStrikes\View::class,
				'dbtech_security_log_login_strikes_view',
				$viewParams
			);
		}

		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\LoginStrike $searcher */
		$searcher = $this->searcher('DBTech\Security:LoginStrike', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\LoginStrikes\Listing::class,
			'dbtech_security_log_login_strikes_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionLoginStrikesSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getLoginStrikeSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\LoginStrikes\Search::class,
			'dbtech_security_log_login_strikes_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 */
	protected function getLoginStrikeSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\LoginStrike $searcher */
		$searcher = $this->searcher('DBTech\Security:LoginStrike');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param int|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\LoginStrike
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertLoginStrikeExists(?int $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\LoginStrike
	{
		return $this->assertRecordExists('DBTech\Security:LoginStrike', $id, $with, $phraseKey);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionChanges(): \XF\Mvc\Reply\AbstractReply
	{
		$page = $this->filterPage();
		$perPage = 20;

		$changeRepo = Helper::repository(\XF\Repository\ChangeLog::class);

		/** @noinspection PhpParamsInspection */
		$changeFinder = $changeRepo->findChangeLogsByContentType([
			'dbtech_security_option',
			'dbtech_security_adminperm',
			'dbtech_security_userperm',
			'dbtech_security_forumperm',
			'dbtech_security_groupperm',
			'dbtech_security_group',
			'user',
		])->limitByPage($page, $perPage);

		$changeFinder->whereOr(
			['content_type', '!=', 'user'],
			[
				['field', [
					'username',
					'password',
					'email',
					'user_group_id',
					'secondary_group_ids'
				]]
			]
		);

		if ($username = $this->filter('username', 'str'))
		{
			$limitUser = Helper::findOne(\XF\Entity\User::class, ['username' => $username]);
			if (!$limitUser)
			{
				return $this->error(\XF::phrase('requested_user_not_found'));
			}
		}
		elseif ($userId = $this->filter('edit_user_id', 'uint'))
		{
			$limitUser = Helper::find(\XF\Entity\User::class, $userId);
			if (!$limitUser)
			{
				return $this->error(\XF::phrase('requested_user_not_found'));
			}
		}
		else
		{
			$limitUser = null;
		}

		$linkFilters = [];
		if ($limitUser)
		{
			$linkFilters['edit_user_id'] = $limitUser->user_id;
			$changeFinder->where('edit_user_id', $limitUser->user_id);
		}

		/** @var \XF\Entity\ChangeLog[]|\XF\Mvc\Entity\ArrayCollection $changes */
		$changes = $changeFinder->fetch();
		$changeRepo->addDataToLogs($changes);

		$viewParams = [
			'changesGrouped' => $changeRepo->groupChangeLogs($changes),
			'totalChanges' => count($changes),
			'limitUser' => $limitUser,

			'page' => $page,
			'perPage' => $perPage,
			'total' => $changeFinder->total(),
			'linkFilters' => $linkFilters
		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\Changes\Listing::class,
			'dbtech_security_log_change_list',
			$viewParams
		);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionCompromisedAccounts(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\CompromisedLog $searcher */
		$searcher = $this->searcher('DBTech\Security:CompromisedLog', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\CompromisedAccounts\Listing::class,
			'dbtech_security_log_compromised_account_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionCompromisedAccountsSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getCompromisedAccountSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\CompromisedAccounts\Search::class,
			'dbtech_security_log_compromised_account_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 */
	protected function getCompromisedAccountSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\CompromisedLog $searcher */
		$searcher = $this->searcher('DBTech\Security:CompromisedLog');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionWatchers(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		if ($params->watcher_log_id)
		{
			$entry = $this->assertWatcherLogExists($params->watcher_log_id, [
				'Watcher',
				'User'
			], 'requested_log_entry_not_found');

			$viewParams = [
				'entry' => $entry,
			];
			return $this->view(
				\DBTech\Security\Admin\View\Log\Watchers\View::class,
				'dbtech_security_log_watcher_view',
				$viewParams
			);
		}

		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\WatcherLog $searcher */
		$searcher = $this->searcher('DBTech\Security:WatcherLog', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$finder->with('Watcher', true);
		$finder->with(['User']);

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\Watchers\Listing::class,
			'dbtech_security_log_watcher_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionWatchersSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getWatcherSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\Watchers\Search::class,
			'dbtech_security_log_watcher_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 * @throws \Exception
	 */
	protected function getWatcherSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\WatcherLog $searcher */
		$searcher = $this->searcher('DBTech\Security:WatcherLog');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param int|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\WatcherLog
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertWatcherLogExists(?int $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\WatcherLog
	{
		return $this->assertRecordExists('DBTech\Security:WatcherLog', $id, $with, $phraseKey);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionFingerprints(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		if ($params->fingerprint_log_id)
		{
			$entry = $this->assertFingerprintLogExists($params->fingerprint_log_id, [
				'User'
			], 'requested_log_entry_not_found');

			$viewParams = [
				'entry' => $entry,
			];
			return $this->view(
				\DBTech\Security\Admin\View\Log\Fingerprints\View::class,
				'dbtech_security_log_fingerprint_view',
				$viewParams
			);
		}

		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\FingerprintLog $searcher */
		$searcher = $this->searcher('DBTech\Security:FingerprintLog', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$finder->with('User');

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\Fingerprints\Listing::class,
			'dbtech_security_log_fingerprint_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionFingerprintsSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getFingerprintSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\Fingerprints\Search::class,
			'dbtech_security_log_fingerprint_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 * @throws \Exception
	 */
	protected function getFingerprintSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\FingerprintLog $searcher */
		$searcher = $this->searcher('DBTech\Security:FingerprintLog');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param int|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\FingerprintLog
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertFingerprintLogExists(?int $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\FingerprintLog
	{
		return $this->assertRecordExists('DBTech\Security:FingerprintLog', $id, $with, $phraseKey);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionBadBehavior(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		if ($params->id)
		{
			$entry = $this->assertBadBehaviorExists($params->id, [
			], 'requested_log_entry_not_found');

			$viewParams = [
				'entry' => $entry,
			];
			return $this->view(
				\DBTech\Security\Admin\View\Log\BadBehavior\View::class,
				'dbtech_security_log_bad_behavior_view',
				$viewParams
			);
		}

		$criteria = $this->filter('criteria', 'array');
		$order = $this->filter('order', 'str');
		$direction = $this->filter('direction', 'str');

		$page = $this->filterPage();
		$perPage = 20;

		/** @var \DBTech\Security\Searcher\BadBehavior $searcher */
		$searcher = $this->searcher('DBTech\Security:BadBehavior', $criteria);

		if ($order && !$direction)
		{
			$direction = $searcher->getRecommendedOrderDirection($order);
		}

		$searcher->setOrder($order, $direction);

		$finder = $searcher->getFinder();
		$finder->limitByPage($page, $perPage);

		$total = $finder->total();
		$entries = $finder->fetch();

		$viewParams = [
			'entries' => $entries,

			'total'   => $total,
			'page'    => $page,
			'perPage' => $perPage,

			'criteria'    => $searcher->getFilteredCriteria(),
			// 'filter' => $filter['text'],
			'sortOptions' => $searcher->getOrderOptions(),
			'order'       => $order,
			'direction'   => $direction

		];
		return $this->view(
			\DBTech\Security\Admin\View\Log\BadBehavior\Listing::class,
			'dbtech_security_log_bad_behavior_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \Exception
	 */
	public function actionBadBehaviorSearch(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getBadBehaviorSearcherParams();

		return $this->view(
			\DBTech\Security\Admin\View\Log\BadBehavior\Search::class,
			'dbtech_security_log_bad_behavior_search',
			$viewParams
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 * @throws \Exception
	 */
	protected function getBadBehaviorSearcherParams(array $extraParams = []): array
	{
		/** @var \DBTech\Security\Searcher\BadBehavior $searcher */
		$searcher = $this->searcher('DBTech\Security:BadBehavior');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}

	/**
	 * @param int|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\BadBehavior
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertBadBehaviorExists(?int $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\BadBehavior
	{
		return $this->assertRecordExists('DBTech\Security:BadBehavior', $id, $with, $phraseKey);
	}
}