<?php

namespace DBTech\Security\Admin\Controller;

use DBTech\Security\Helper;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

/**
 * Class Password
 * @package DBTech\Security\Admin\Controller
 */
class Password extends AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('dbtechSecurity');
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionForceChange(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getSearcherParams(['success' => $this->filter('success', 'bool')]);
		return $this->view(
			\DBTech\Security\Admin\View\Password\Reset::class,
			'dbtech_security_password_change',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionForceReset(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = $this->getSearcherParams(['success' => $this->filter('success', 'bool')]);
		return $this->view(
			\DBTech\Security\Admin\View\Password\Reset::class,
			'dbtech_security_password_reset',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionGenerator(): \XF\Mvc\Reply\AbstractReply
	{
		if ($this->isPost())
		{
			$passwordRepo = Helper::repository(\DBTech\Security\Repository\Password::class);

			$generatedPassword = '';
			if ($this->filter('password_chooser', 'bool'))
			{
				// Generate password
				$generatedPassword = $passwordRepo->generatePassword(
					$this->filter('length', 'uint'),
					$this->filter('rules', 'array-bool')
				);

				$key = $this->filter('username', 'str') . ':' . $passwordRepo->encryptPasswordForBasicAuth($generatedPassword);
			}
			else
			{
				$key = $this->filter('username', 'str') . ':' . $passwordRepo->encryptPasswordForBasicAuth(
					$this->filter('password', 'str')
				);
			}

			$viewParams = [
				'password' => $generatedPassword,
				'key' => $key
			];
			return $this->view(
				\DBTech\Security\Admin\View\Password\Generator\Result::class,
				'dbtech_security_password_generator_result',
				$viewParams
			);
		}

		return $this->view(
			\DBTech\Security\Admin\View\Password\Generator\Form::class,
			'dbtech_security_password_generator'
		);
	}

	/**
	 * @param array $extraParams
	 *
	 * @return array
	 */
	protected function getSearcherParams(array $extraParams = []): array
	{
		$searcher = $this->searcher('XF:User');

		$viewParams = [
			'criteria'   => $searcher->getFormCriteria(),
			'sortOrders' => $searcher->getOrderOptions()
		];
		return $viewParams + $searcher->getFormData() + $extraParams;
	}
}