<?php

namespace DBTech\Security\Admin\Controller;

use DBTech\Security\Helper;
use XF\Admin\Controller\AbstractController;
use XF\Mvc\ParameterBag;

/**
 * Class Snapshot
 * @package DBTech\Security\Admin\Controller
 */
class Snapshot extends AbstractController
{
	/**
	 * @param $action
	 * @param ParameterBag $params
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function preDispatchController($action, ParameterBag $params)
	{
		$this->assertAdminPermission('dbtechSecurity');
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionIndex(): \XF\Mvc\Reply\AbstractReply
	{
		$viewParams = [
			'snapshots' => Helper::repository(\DBTech\Security\Repository\Backup::class)
				->findSnapshotsForList()
				->fetch()
		];
		return $this->view(
			\DBTech\Security\Admin\View\Snapshot\Listing::class,
			'dbtech_security_snapshot_list',
			$viewParams
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\PrintableException
	 */
	public function actionBackup(): \XF\Mvc\Reply\AbstractReply
	{
		$snapshotId = Helper::repository(\DBTech\Security\Repository\Backup::class)
			->backupOptions()
		;

		return $this->redirect(
			$this->buildLink('dbtech-security/settings-backups') . $this->buildLinkHash($snapshotId),
			\XF::phrase('dbtech_security_backup_saved')
		);
	}

	/**
	 * @param \XF\Mvc\ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionLoad(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		$snapshot = $this->assertSnapshotExists($params->snapshot_id);

		if ($this->isPost())
		{
			if ($snapshot->data)
			{
				Helper::repository(\DBTech\Security\Repository\Backup::class)
					->updateOptions($snapshot->data)
				;
			}

			return $this->redirect(
				$this->buildLink('dbtech-security/settings-backups'),
				\XF::phrase('dbtech_security_backup_loaded')
			);
		}

		$viewParams = [
			'snapshot' => $snapshot
		];
		return $this->view(
			\DBTech\Security\Admin\View\Snapshot\Load::class,
			'dbtech_security_snapshot_load',
			$viewParams
		);
	}

	/**
	 * @param ParameterBag $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionDelete(ParameterBag $params): \XF\Mvc\Reply\AbstractReply
	{
		$snapshot = $this->assertSnapshotExists($params->snapshot_id);

		/** @var \XF\ControllerPlugin\Delete $plugin */
		$plugin = $this->plugin('XF:Delete');
		return $plugin->actionDelete(
			$snapshot,
			$this->buildLink('dbtech-security/settings-backups/delete', $snapshot),
			null,
			$this->buildLink('dbtech-security/settings-backups'),
			$snapshot->title
		);
	}

	/**
	 * @param string|null $id
	 * @param array|string|null $with
	 * @param null|string $phraseKey
	 *
	 * @return \DBTech\Security\Entity\Snapshot
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertSnapshotExists(?string $id, $with = null, ?string $phraseKey = null): \DBTech\Security\Entity\Snapshot
	{
		return $this->assertRecordExists('DBTech\Security:Snapshot', $id, $with, $phraseKey);
	}
}