<?php

namespace DBTech\Security\ChangeLog;

use DBTech\Security\Helper;
use XF\ChangeLog\AbstractHandler;

/**
 * Class AbstractPermission
 * @package DBTech\Security\ChangeLog
 */
abstract class AbstractPermission extends AbstractHandler
{
	protected ?array $permissionsGrouped = null;

	/**
	 * @return array
	 */
	protected function getLabelMap(): ?array
	{
		return $this->getPermissions();
	}

	/**
	 * @return array
	 */
	protected function getFormatterMap(): array
	{
		$permissionsGrouped = $this->getPermissions();

		return array_fill_keys(array_keys($permissionsGrouped), 'formatPermission');
	}

	/**
	 * @return array
	 */
	protected function getPermissions(): ?array
	{
		if ($this->permissionsGrouped === null)
		{
			/** @var \XF\Repository\Permission $permissionRepo */
			$permissionRepo = Helper::repository(\XF\Repository\Permission::class);
			$grouped = $permissionRepo->getPermissionsGrouped();

			$this->permissionsGrouped = [];
			foreach ($grouped as $group => $permissions)
			{
				/** @var \XF\Entity\Permission $permission */
				foreach ($permissions as $permissionId => $permission)
				{
					$this->permissionsGrouped[$group . '_' . $permissionId] = $permission->getPhraseName();
				}
			}
		}

		return $this->permissionsGrouped;
	}

	/**
	 * @param string $permission
	 *
	 * @return string
	 */
	protected function formatPermission(string $permission): string
	{
		switch ($permission)
		{
			case 'unset':
			case 'allow':
			case 'deny':
			case 'content_allow':
			case 'reset':
				return \XF::phrase('dbtech_security_' . $permission);

			default:
				return $permission;
		}
	}
}