<?php

namespace DBTech\Security\ControllerPlugin;

use DBTech\Security\Helper;
use XF\ControllerPlugin\AbstractPlugin;
use XF\Service\AbstractService;

/**
 * Class Delete
 *
 * @package DBTech\Security\ControllerPlugin
 */
class Delete extends AbstractPlugin
{
	/**
	 * @template T of AbstractService
	 * @param \XF\Mvc\Entity\Entity $entity
	 * @param string $stateKey
	 * @param class-string<T> $deleterService
	 * @param string $contentType
	 * @param string $deleteLink
	 * @param string $editLink
	 * @param string $redirectLink
	 * @param \XF\Entity\Phrase|string $title
	 * @param bool $canHardDelete
	 * @param bool $includeAuthorAlert
	 * @param string|null $templateName
	 * @param array $params
	 *
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @noinspection PhpMissingParamTypeInspection
	 */
	public function actionDeleteWithState(
		\XF\Mvc\Entity\Entity $entity,
		string $stateKey,
		string $deleterService,
		string $contentType,
		string $deleteLink,
		string $editLink,
		string $redirectLink,
		$title,
		bool $canHardDelete = false,
		bool $includeAuthorAlert = true,
		?string $templateName = null,
		array $params = []
	): \XF\Mvc\Reply\AbstractReply {
		if ($this->isPost())
		{
			$id = $entity->getIdentifierValues();
			if (!$id || count($id) != 1)
			{
				throw new \InvalidArgumentException("Entity does not have an ID or does not have a simple key");
			}
			$entityId = intval(reset($id));

			if ($entity->{$stateKey} == 'deleted')
			{
				$linkHash = $this->buildLinkHash($entityId);

				$type = $this->filter('hard_delete', 'uint');
				switch ($type)
				{
					case 0:
						return $this->redirect($redirectLink . $linkHash);

					case 1:
						$reason = $this->filter('reason', 'str');

						$deleter = Helper::service($deleterService, $entity);
						if ($includeAuthorAlert && $this->filter('author_alert', 'bool'))
						{
							$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
						}
						$deleter->delete('hard', $reason);

						/** @var \XF\ControllerPlugin\InlineMod $inlineModPlugin */
						$inlineModPlugin = $this->plugin('XF:InlineMod');
						$inlineModPlugin->clearIdFromCookie($contentType, $entityId);

						return $this->redirect($redirectLink);

					case 2:
						$deleter = Helper::service($deleterService, $entity);
						if ($includeAuthorAlert && $this->filter('author_alert', 'bool'))
						{
							$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
						}
						$deleter->unDelete();

						return $this->redirect($redirectLink . $linkHash);
				}
			}
			else
			{
				$type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
				$reason = $this->filter('reason', 'str');

				$deleter = Helper::service($deleterService, $entity);
				if ($includeAuthorAlert && $this->filter('author_alert', 'bool'))
				{
					$deleter->setSendAlert(true, $this->filter('author_alert_reason', 'str'));
				}
				$deleter->delete($type, $reason);

				/** @var \XF\ControllerPlugin\InlineMod $inlineModPlugin */
				$inlineModPlugin = $this->plugin('XF:InlineMod');
				$inlineModPlugin->clearIdFromCookie($contentType, $entityId);

				return $this->redirect($redirectLink);
			}
		}

		$templateName = $templateName ?: 'public:dbtech_security_delete_state';
		$viewClass = $this->app->get('app.classType') == 'Pub'
			? \DBTech\Security\Pub\View\Delete\State::class
			: \DBTech\Security\Admin\View\Delete\State::class
		;

		$viewParams = [
				'entity'             => $entity,
				'stateKey'           => $stateKey,
				'title'              => $title,
				'editLink'           => $editLink,
				'deleteLink'         => $deleteLink,
				'canHardDelete'      => $canHardDelete,
				'includeAuthorAlert' => $includeAuthorAlert
			] + $params;
		return $this->view($viewClass, $templateName, $viewParams);
	}
}