<?php

namespace DBTech\Security\ControllerPlugin;

use DBTech\Security\Helper;
use XF\ControllerPlugin\AbstractPlugin;

/**
 * Class Login
 *
 * @package DBTech\Security\ControllerPlugin
 */
class Login extends AbstractPlugin
{
	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 */
	public function actionSecurityKeyConfirm(): \XF\Mvc\Reply\AbstractReply
	{
		$redirect = $this->controller->getDynamicRedirectIfNot($this->buildLink('login'));
		$visitor = \XF::visitor();

		if (!$visitor->user_id)
		{
			return $this->redirect($redirect, '');
		}

		$this->assertPostOnly();

		$providerId = 'dbtech_security_authn';

		/** @var \XF\Service\User\Tfa $tfaService */
		$tfaService = Helper::service(\XF\Service\User\Tfa::class, $visitor);
		if (!$tfaService->isProviderValid($providerId))
		{
			return $this->error(
				\XF::phrase('dbtech_security_security_key_not_registered_please_use_password')
			);
		}

		if ($tfaService->hasTooManyTfaAttempts())
		{
			return $this->error(
				\XF::phrase('your_account_has_temporarily_been_locked_due_to_failed_login_attempts')
			);
		}

		$verified = $tfaService->verify($this->request, $providerId);
		if (!$verified)
		{
			return $this->error(\XF::phrase('two_step_verification_value_could_not_be_confirmed'));
		}
		else
		{
			/** @var \XF\ControllerPlugin\Login $loginPlugin */
			$loginPlugin = $this->plugin('XF:Login');
			$loginPlugin->clearTfaSessionCheck();

			$this->session()->passwordConfirm = \XF::$time;
			return $this->redirect($redirect, '');
		}
	}
}