<?php

namespace DBTech\Security\Install;

use XF\Db\Schema\Alter;

/**
 * @property \XF\AddOn\AddOn addOn
 * @property \XF\App app
 *
 * @method \XF\Db\AbstractAdapter db()
 * @method \XF\Db\SchemaManager schemaManager()
 */
trait UninstallDataTrait
{
	/**
	 * Methods MUST start at step 4, as steps 1-3 are reserved by the core
	 */
	
	protected function runMiscCleanUp(): void
	{
		// Get rid of change logs
		$this->db()->delete('xf_change_log', "content_type LIKE 'dbtech_security_%'");
		$this->db()->delete('xf_change_log', "field LIKE 'dbtech_security_%'");

		$this->db()->delete('xf_tfa_provider', "provider_id = 'dbtech_security_authn'");

		$contentTypesQuoted = $this->db()->quote([
			'dbtech_security_tor',
			'dbtech_security_country'
		]);
		$this->db()->delete('xf_ip_match', 'match_type IN (' . $contentTypesQuoted . ')');
	}

	/**
	 *
	 */
	public function uninstallStep4()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_ip_match', function (Alter $table)
		{
			$table->changeColumn('match_type')->removeValues(['dbtech_security_tor', 'dbtech_security_country']);
		});
	}
}