<?php

namespace DBTech\Security\Install;

use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

/**
 * @property \XF\AddOn\AddOn addOn
 * @property \XF\App app
 *
 * @method \XF\Db\AbstractAdapter db()
 * @method \XF\Db\SchemaManager schemaManager()
 * @method \XF\Db\Schema\Column addOrChangeColumn($table, $name, $type = null, $length = null)
 */
trait Upgrade904019970Trait
{
	/**
	 *
	 */
	public function upgrade904010031Step1()
	{
		$sm = $this->schemaManager();

		$sm->dropTable('xf_dbtech_security_ipbanlog');
		$sm->dropTable('xf_dbtech_security_tornode');

		$sm->renameTable('xf_dbtech_security_adminstrikes', 'xf_dbtech_security_admin_strike');
		$sm->renameTable('xf_dbtech_security_badbehavior', 'xf_dbtech_security_bad_behavior');
		$sm->renameTable('xf_dbtech_security_changelog', 'xf_dbtech_security_change_log');
		$sm->renameTable('xf_dbtech_security_compromisedlog', 'xf_dbtech_security_compromised_log');
		$sm->renameTable('xf_dbtech_security_fingerprintlog', 'xf_dbtech_security_fingerprint_log');
		$sm->renameTable('xf_dbtech_security_iplog', 'xf_dbtech_security_ip_log');
		$sm->renameTable('xf_dbtech_security_ipverify', 'xf_dbtech_security_ip_verify');
		$sm->renameTable('xf_dbtech_security_loginstrikes', 'xf_dbtech_security_login_strike');
		$sm->renameTable('xf_dbtech_security_recoverylog', 'xf_dbtech_security_recovery_log');
		$sm->renameTable('xf_dbtech_security_watcherlog', 'xf_dbtech_security_watcher_log');

		$tables = $this->getTables();

		$key = 'xf_dbtech_security_country';
		$sm->createTable($key, $tables[$key]);
	}

	/**
	 *
	 * @throws \XF\Db\Exception
	 */
	public function upgrade904010031Step2()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_dbtech_security_fingerprint_log', function (Alter $table)
		{
			$table->dropPrimaryKey();
		});

		$this->query('
			ALTER TABLE xf_dbtech_security_fingerprint_log
			ADD fingerprint_log_id INT(10) UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY
				FIRST
		');
	}

	/**
	 *
	 */
	public function upgrade904010031Step3()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_dbtech_security_admin_strike', function (Alter $table)
		{
			$table->renameColumn('adminstrikesid', 'admin_strike_id');
			$table->renameColumn('userid', 'user_id');
		});

		$sm->alterTable('xf_dbtech_security_change_log', function (Alter $table)
		{
			$table->renameColumn('changelogid', 'change_log_id');
			$table->renameColumn('userid', 'user_id');
		});

		$sm->alterTable('xf_dbtech_security_compromised_log', function (Alter $table)
		{
			$table->renameColumn('compromisedlogid', 'compromised_log_id');
			$table->renameColumn('userid', 'user_id');
			$table->renameColumn('attemptedusernames', 'attempted_usernames');
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step4()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_dbtech_security_criteria', function (Alter $table)
		{
			$table->renameColumn('criteriaid', 'criteria_id');
		});

		$sm->alterTable('xf_dbtech_security_fingerprint_log', function (Alter $table)
		{
			$table->renameColumn('userid', 'user_id');
			$table->addUniqueKey(['fingerprint', 'user_id'], 'fingerprint_user_id');
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step5()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_dbtech_security_ip_log', function (Alter $table)
		{
			$table->renameColumn('userid', 'user_id');
			$table->renameColumn('firstvisit', 'first_visit');
			$table->renameColumn('lastvisit', 'last_visit');
		});

		$sm->alterTable('xf_dbtech_security_ip_verify', function (Alter $table)
		{
			$table->renameColumn('ipverifyid', 'ip_verify_id');
			$table->renameColumn('userid', 'user_id');
		});

		$sm->alterTable('xf_dbtech_security_login_strike', function (Alter $table)
		{
			$table->renameColumn('loginstrikesid', 'login_strike_id');
			$table->renameColumn('validuser', 'valid_user');
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step6()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_dbtech_security_recovery_log', function (Alter $table)
		{
			$table->renameColumn('recoverylogid', 'recovery_log_id');
			$table->renameColumn('userid', 'user_id');
			$table->renameColumn('existingemail', 'existing_email');
			$table->renameColumn('newemail', 'new_email');
			$table->renameColumn('pmuser', 'pm_user');
		});

		$sm->alterTable('xf_dbtech_security_snapshot', function (Alter $table)
		{
			$table->renameColumn('snapshotid', 'snapshot_id');
		});

		$sm->alterTable('xf_dbtech_security_watcher_log', function (Alter $table)
		{
			$table->renameColumn('watcherlogid', 'watcher_log_id');
			$table->renameColumn('dateline', 'log_date');
			$table->renameColumn('ipaddress', 'ip_address');
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step7()
	{
		$sm = $this->schemaManager();

		$this->db()->delete('xf_dbtech_security_watcher_log', null);

		$sm->alterTable('xf_dbtech_security_watcher_log', function (Alter $table)
		{
			$table->addColumn('log_date', 'int')->setDefault(0)->after('watcher_log_id');
			$table->addColumn('user_id', 'int')->setDefault(0)->after('log_date');
			$table->addColumn('watcher_id', 'int')->setDefault(0)->after('ip_address');
			$table->addColumn('action_params', 'mediumblob')->nullable(true)->after('action');
		});
	}

	public function upgrade904010031Step8()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_user_remember', function (Alter $table)
		{
			$table->addColumn('dbtech_security_user_agent', 'text')->nullable(true);
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step9()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_ip_match', function (Alter $table)
		{
			$table->changeColumn('match_type')->addValues(['dbtech_security_tor', 'dbtech_security_country']);
			$table->changeColumn('dbtech_security_comment')->resetDefinition()->type('varchar', 15)->setDefault('');
		});
	}

	/**
	 *
	 */
	public function upgrade904010031Step10()
	{
		$sm = $this->schemaManager();

		$sm->renameTable('xf_dbtech_security_watcher', 'xf_dbtech_security_watcher_temp');

		$sm->createTable('xf_dbtech_security_watcher', function (Create $table)
		{
			$table->addColumn('watcher_id', 'int')->autoIncrement();
			$table->addColumn('watcher_type', 'varchar', 50)->setDefault('');
			$table->addColumn('active', 'tinyint')->setDefault(1);
			$table->addColumn('priority', 'int')->setDefault(1);
			$table->addColumn('rule_data', 'mediumblob')->nullable(true);
			$table->addColumn('actions', 'mediumblob')->nullable(true);
			$table->addColumn('extra_data', 'mediumblob')->nullable(true);
		});

		$oldWatchers = $this->db()->fetchAll("
			SELECT *
			FROM xf_dbtech_security_watcher_temp
		");

		$insert = [];
		foreach ($oldWatchers as $oldWatcher)
		{
			switch ($oldWatcher['watcher'])
			{
				case 'username':
				case 'password':
				case 'email':
				case 'usergroupid':
				case 'membergroupids':
					$watcherType = 'user';
					break;

				case 'boardActive':
				case 'boardInactiveMessage':
				case 'dbtech_security_allowip':
				case 'dbtech_security_allowip_exclude':
					$watcherType = 'option';
					break;

				default:
					$watcherType = $oldWatcher['watcher'];
					break;
			}
			if (!$watcherType)
			{
				continue;
			}

			$oldWatcher['ruledata'] = \XF\Util\Php::safeUnserialize($oldWatcher['ruledata']);

			switch ($oldWatcher['watcher'])
			{
				case 'configtamper':
					$oldWatcher['extradata'] = $oldWatcher['extradata'] ? ['differences' => $oldWatcher['extradata']] : [];
					break;

				default:
					$oldWatcher['extradata'] = \XF\Util\Php::safeUnserialize($oldWatcher['extradata']);
					break;
			}

			$ruleData = is_array($oldWatcher['ruledata']) ? $oldWatcher['ruledata'] : [];

			if (isset($ruleData['ipaddresses']))
			{
				$ruleData['ipaddresses'] = $ruleData['ipaddresses'] ? 'same' : 'any';
			}

			switch ($watcherType)
			{
				case 'user':
					switch ($oldWatcher['watcher'])
					{
						case 'usergroupid':
							$ruleData['field'] = 'user_group_id';
							break;

						case 'membergroupids':
							$ruleData['field'] = 'secondary_group_ids';
							break;

						default:
							$ruleData['field'] = $oldWatcher['watcher'];
							break;
					}
					break;

				case 'option':
					$ruleData['field'] = $oldWatcher['watcher'];
			}

			if (isset($ruleData['intrusions']) && !$ruleData['intrusions'])
			{
				continue;
			}

			if (isset($ruleData['hours']) && !$ruleData['hours'])
			{
				continue;
			}

			$insert[] = [
				'watcher_type' => $watcherType,
				'active' => 1,
				'priority' => ($oldWatcher['ruleid'] + 1),
				'rule_data' => json_encode($ruleData),
				'extra_data' => json_encode(is_array($oldWatcher['extradata']) ? $oldWatcher['extradata'] : []),
				'actions' => json_encode([
					'closeForum' => (bool)((int)$oldWatcher['actions'] & 1),
					'emailWebmaster' => (bool)((int)$oldWatcher['actions'] & 2),
					'banUser' => (bool)((int)$oldWatcher['actions'] & 4),
					'banIp' => (bool)((int)$oldWatcher['actions'] & 8),
					'emailUser' => (bool)((int)$oldWatcher['actions'] & 16),
					'lockUser' => (bool)((int)$oldWatcher['actions'] & 32),
					'adminLockUser' => (bool)((int)$oldWatcher['actions'] & 64)
				])
			];
		}

		if (count($insert))
		{
			$this->db()->insertBulk('xf_dbtech_security_watcher', $insert, true);
		}

		$sm->dropTable('xf_dbtech_security_watcher_temp');
	}

	/**
	 *
	 * @throws \XF\Db\Exception
	 */
	public function upgrade904010031Step11()
	{
		$globalAllowIp = $this->db()->fetchOne("
			SELECT option_value
			FROM xf_option
			WHERE option_id = 'dbtech_security_globalallowip'
		");

		$newOptionValue = [];

		$ips = preg_split('#\s+#', $globalAllowIp, -1, PREG_SPLIT_NO_EMPTY);
		foreach ($ips as $ip)
		{
			$parsed = \XF\Util\Ip::parseIpRangeString($ip);

			if (!$parsed)
			{
				continue;
			}

			$newOptionValue[] = [
				'ip' => $ip,
				'isRange' => $parsed['isRange'],
			];
		}

		$this->query("
			UPDATE xf_option
			SET option_value = ?
			WHERE option_id = 'dbtech_security_globalallowip'
		", json_encode($newOptionValue));
	}

	/**
	 *
	 * @throws \XF\Db\Exception
	 */
	public function upgrade904010031Step12()
	{
		$allowIp = $this->db()->fetchOne("
			SELECT option_value
			FROM xf_option
			WHERE option_id = 'dbtech_security_allowip'
		");

		$newOptionValue = [];

		$ips = preg_split('#\s+#', $allowIp, -1, PREG_SPLIT_NO_EMPTY);
		foreach ($ips as $ip)
		{
			$parsed = \XF\Util\Ip::parseIpRangeString($ip);

			if (!$parsed)
			{
				continue;
			}

			$newOptionValue[] = [
				'ip' => $ip,
				'isRange' => $parsed['isRange'],
			];
		}

		$this->query("
			UPDATE xf_option
			SET option_value = ?
			WHERE option_id = 'dbtech_security_allowip'
		", json_encode($newOptionValue));
	}

	public function upgrade904060970Step1()
	{
		$sm = $this->schemaManager();

		$sm->alterTable('xf_user_remember', function (Alter $table)
		{
			$table->changeColumn('dbtech_security_user_agent')
				->resetDefinition()
				->type('blob')
				->nullable(true)
			;
		});
	}

	/**
	 * @param $previousVersion
	 * @param array $stateChanges
	 *
	 * @throws \XF\PrintableException
	 */
	protected function postUpgrade904010031($previousVersion, array &$stateChanges)
	{
		$torRepo = \XF::repository('DBTech\Security:Tor');
		$torRepo->updateNodes();

		$countryRepo = \XF::repository('DBTech\Security:Country');
		$countryRepo->updateCountryList();

		$this->app->jobManager()->enqueueUnique(
			'dbtechSecurityCountryBlockRebuild',
			'DBTech\Security:CountryBlockRebuild',
			[],
			false
		);
	}
}