<?php

namespace DBTech\Security\Job;

use DBTech\Security\Helper;
use XF\Job\AbstractJob;

/**
 * Class CountryBlockRebuild
 *
 * @package DBTech\Security\Job
 */
class CountryBlockRebuild extends AbstractJob
{
	/** @var array  */
	protected $defaultData = [
		'steps' => 0,
		'countryCodes' => [],
		'cleaned' => false
	];


	/**
	 * @param int $maxRunTime
	 *
	 * @return \XF\Job\JobResult
	 * @throws \XF\PrintableException
	 */
	public function run($maxRunTime): \XF\Job\JobResult
	{
		$start = microtime(true);

		if (!$this->data['cleaned'])
		{
			$countryRepo = Helper::repository(\DBTech\Security\Repository\Country::class);

			$banningRepo = Helper::repository(\DBTech\Security\Repository\Banning::class);

			$countries = $countryRepo->findCountriesForList()
				->fetch()
			;

			/** @var \DBTech\Security\Entity\Country $country */
			foreach ($countries as $country)
			{
				// Remove existing countries
				$banningRepo->removeBannedCountryIps($country->country_code);

				if ($country->blocked)
				{
					$this->data['countryCodes'][$country->country_code] = true;
				}
			}

			$this->data['cleaned'] = true;
		}

		if (empty($this->data['countryCodes']))
		{
			return $this->complete();
		}

		$this->data['steps']++;

		$db = $this->app->db();
		$countryCodes = $db->fetchAllColumn("
			SELECT country_code
			FROM xf_dbtech_security_country
			WHERE country_code IN(" . $db->quote(array_keys($this->data['countryCodes'])) . ")
			ORDER BY country_code
		");
		if (!$countryCodes)
		{
			return $this->complete();
		}

		$banningRepo = Helper::repository(\DBTech\Security\Repository\Banning::class);

		foreach ($countryCodes AS $countryCode)
		{
			$banningRepo->updateAndBanIpsForCountry($countryCode);
			unset($this->data['countryCodes'][$countryCode]);

			if (microtime(true) - $start >= $maxRunTime)
			{
				break;
			}
		}

		return $this->resume();
	}

	/**
	 * @return string
	 */
	public function getStatusMessage(): string
	{
		$actionPhrase = \XF::phrase('rebuilding');
		$typePhrase = \XF::phrase('dbtech_security_country_blocks');
		return sprintf('%s... %s %s', $actionPhrase, $typePhrase, str_repeat('. ', $this->data['steps']));
	}

	/**
	 * @return bool
	 */
	public function canCancel(): bool
	{
		return false;
	}

	/**
	 * @return bool
	 */
	public function canTriggerByChoice(): bool
	{
		return false;
	}
}