<?php

namespace DBTech\Security\Pub\Controller;

use DBTech\Security\Helper;
use XF\Pub\Controller\AbstractController;

/**
 * Class AccountLock
 *
 * @package DBTech\Security\Pub\Controller
 */
class AccountLock extends AbstractController
{
	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionIndex(): \XF\Mvc\Reply\AbstractReply
	{
		$this->assertRegistrationRequired();
		$this->assertIsLocked();

		return $this->view(
			\DBTech\Security\Pub\View\AccountLock\Index::class,
			'dbtech_security_locked_account'
		);
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionResend(): \XF\Mvc\Reply\AbstractReply
	{
		$this->assertRegistrationRequired();
		$this->assertIsUserLocked();

		$watcherService = Helper::service(\DBTech\Security\Service\Watcher\Trigger::class, null, \XF::visitor());
		$watcherService->actionLockAccount('user');

		return $this->message(\XF::phrase('dbtech_security_unlock_code_sent'));
	}

	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 * @throws \XF\PrintableException
	 */
	public function actionUnlock(): \XF\Mvc\Reply\AbstractReply
	{
		$userId = $this->filter('user_id', 'uint');
		$hash = $this->filter('hash', 'str');

		$lockedUser = $this->assertValidatedUnlockHash($userId, $hash);

		Helper::repository(\DBTech\Security\Repository\Watcher::class)
			->unlockAccount($lockedUser)
		;

		return $this->message(\XF::phrase('dbtech_security_account_unlocked'));
	}

	/**
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertIsLocked()
	{
		/** @var \DBTech\Security\XF\Entity\User $visitor */
		$visitor = \XF::visitor();

		if (!$visitor->Option->dbtech_security_is_user_locked
			&& !$visitor->Option->dbtech_security_is_admin_locked
		) {
			throw $this->exception($this->message(\XF::phrase('dbtech_security_not_locked'), 403));
		}
	}

	/**
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertIsUserLocked()
	{
		$this->assertIsLocked();

		/** @var \DBTech\Security\XF\Entity\User $visitor */
		$visitor = \XF::visitor();

		if ($visitor->Option->dbtech_security_is_admin_locked)
		{
			throw $this->exception($this->message(\XF::phrase('dbtech_security_account_can_only_be_admin_unlocked'), 403));
		}
	}

	/**
	 * @param int|null $userId
	 * @param string $confirmKey
	 *
	 * @return \XF\Entity\User
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertValidatedUnlockHash(?int $userId, string $confirmKey): \XF\Entity\User
	{
		$user = Helper::find(\XF\Entity\User::class, $userId);
		if (!$user)
		{
			throw $this->exception(
				$this->error(\XF::phrase('this_link_is_not_usable_by_you'), 403)
			);
		}

		if (!$user->DBTechSecurityAccountLock || $confirmKey !== $user->DBTechSecurityAccountLock->hash)
		{
			throw $this->exception(
				$this->error(\XF::phrase('this_link_could_not_be_verified'), 403)
			);
		}

		if ($user->DBTechSecurityAccountLock->action == 'admin'
			&& !\XF::visitor()->is_admin
		) {
			throw $this->exception(
				$this->error(\XF::phrase('this_link_is_not_usable_by_you'), 403)
			);
		}

		return $user;
	}
}