<?php

namespace DBTech\Security\Repository;

use DBTech\Security\Helper;
use XF\Mvc\Entity\Finder;
use XF\Mvc\Entity\Repository;

/**
 * Class Backup
 * @package DBTech\Security\Repository
 */
class Backup extends Repository
{
	/**
	 * @return Finder
	 */
	public function findSnapshotsForList(): Finder
	{
		return Helper::finder(\DBTech\Security\Finder\Snapshot::class)
			->order('dateline', 'DESC')
		;
	}

	/**
	 * @return int
	 * @throws \XF\PrintableException
	 */
	public function backupOptions(): int
	{
		$snapshot = Helper::createEntity(\DBTech\Security\Entity\Snapshot::class);
		$snapshot->data = Helper::repository(\XF\Repository\Option::class)->getOptionCacheData();
		$snapshot->save();

		// Insert the settings backup
		$this->db()->delete('xf_dbtech_security_snapshot', 'dateline <= ?', \XF::$time - 604800);

		return $snapshot->snapshot_id;
	}

	/**
	 * @param array $data
	 */
	public function updateOptions(array $data)
	{
		$db = $this->db();

		$db->beginTransaction();

		foreach ($data as $key => $value)
		{
			$db->update('xf_option', [
				'option_value' => is_array($value) ? json_encode($value) : $value
			], 'option_id = ?', $key);
		}

		$db->commit();
	}
}