<?php

namespace DBTech\Security\WatcherLog;

use DBTech\Security\Entity\Watcher;
use DBTech\Security\Helper;

class Logger
{
	/**
	 * @param Watcher $content
	 * @param string $action
	 * @param string $message
	 * @param array $params
	 * @param bool $throw
	 * @param \XF\Entity\User|null $actor
	 *
	 * @return \DBTech\Security\Entity\WatcherLog
	 * @throws \XF\PrintableException
	 */
	public function log(
		Watcher $content,
		string $action,
		string $message,
		array $params = [],
		bool $throw = true,
		?\XF\Entity\User $actor = null
	): ?\DBTech\Security\Entity\WatcherLog {
		if ($this->isLoggableUser($content, $action, $actor) === null)
		{
			return null;
		}

		$log = Helper::createEntity(\DBTech\Security\Entity\WatcherLog::class);
		$log->log_date = \XF::$time;
		if ($this->isLoggableUser($content, $action, $actor))
		{
			$log->user_id = $actor->user_id;
		}
		$log->ip_address = !empty($params['ip_address']) ? $params['ip_address'] : '';
		$log->watcher_id = $content->watcher_id;
		$log->action = $action;
		$log->action_params = $params;
		$log->message = $message;
		$log->save($throw);

		return $log;
	}

	/**
	 * @param $content
	 * @param $action
	 * @param \XF\Entity\User|null $actor
	 *
	 * @return bool|int|null
	 */
	public function isLoggableUser($content, $action, ?\XF\Entity\User $actor = null)
	{
		if ($action == 'ban_user')
		{
			if ($actor === null)
			{
				return null;
			}

			return $actor->user_id;
		}

		return false;
	}
}