<?php
namespace DBTech\Security\WebAuthn;

use Webauthn\PublicKeyCredentialSourceRepository as PublicKeyCredentialSourceRepositoryInterface;
use Webauthn\PublicKeyCredentialSource;
use Webauthn\PublicKeyCredentialUserEntity;

class PublicKeyCredentialSourceRepositoryDb implements PublicKeyCredentialSourceRepositoryInterface
{
	private array $credentials;


	/**
	 * PublicKeyCredentialSourceRepositoryDb constructor.
	 *
	 * @param array $credentials
	 */
	public function __construct(array $credentials = [])
	{
		$this->credentials = $credentials;
	}

	/**
	 * @param string $publicKeyCredentialId
	 *
	 * @return PublicKeyCredentialSource|null
	 */
	public function findOneByCredentialId(string $publicKeyCredentialId): ?PublicKeyCredentialSource
	{
		$data = $this->read();

		$encodedCredentialId = base64_encode($publicKeyCredentialId);
		if (isset($data[$encodedCredentialId]))
		{
			return PublicKeyCredentialSource::createFromArray($data[$encodedCredentialId]['device']);
		}
		return null;
	}

	/**
	 * @param PublicKeyCredentialUserEntity $publicKeyCredentialUserEntity
	 *
	 * @return PublicKeyCredentialSource[]
	 */
	public function findAllForUserEntity(PublicKeyCredentialUserEntity $publicKeyCredentialUserEntity): array
	{
		$sources = [];
		foreach ($this->read() as $data)
		{
			$source = PublicKeyCredentialSource::createFromArray($data['device']);
			if ($source->getUserHandle() === $publicKeyCredentialUserEntity->getId())
			{
				$sources[] = $source;
			}
		}
		return $sources;
	}

	/**
	 * @param PublicKeyCredentialSource $publicKeyCredentialSource
	 */
	public function saveCredentialSource(PublicKeyCredentialSource $publicKeyCredentialSource): void
	{
	}

	/**
	 * @return array
	 */
	private function read(): array
	{
		return $this->credentials;
	}
}