<?php
namespace DBTech\Security\WebAuthn;

use Webauthn\PublicKeyCredentialSourceRepository as PublicKeyCredentialSourceRepositoryInterface;
use Webauthn\PublicKeyCredentialSource;
use Webauthn\PublicKeyCredentialUserEntity;

class PublicKeyCredentialSourceRepositoryFile implements PublicKeyCredentialSourceRepositoryInterface
{
	private string $path;


	/**
	 * PublicKeyCredentialSourceRepository constructor.
	 */
	public function __construct()
	{
		$createDir = \XF::app()->config('internalDataPath') . DIRECTORY_SEPARATOR . 'dbtechSecurity';
		$this->path = $createDir . DIRECTORY_SEPARATOR . 'pubkey-repo.json';

		try
		{
			\XF::app()->fs()->createDir($createDir);
		} /** @noinspection PhpRedundantCatchClauseInspection */
		catch (\League\Flysystem\FileNotFoundException $e)
		{
		}
	}

	/**
	 * @param string $publicKeyCredentialId
	 *
	 * @return PublicKeyCredentialSource|null
	 */
	public function findOneByCredentialId(string $publicKeyCredentialId): ?PublicKeyCredentialSource
	{
		$data = $this->read();
		if (isset($data[base64_encode($publicKeyCredentialId)]))
		{
			return PublicKeyCredentialSource::createFromArray($data[base64_encode($publicKeyCredentialId)]);
		}
		return null;
	}

	/**
	 * @param PublicKeyCredentialUserEntity $publicKeyCredentialUserEntity
	 *
	 * @return PublicKeyCredentialSource[]
	 */
	public function findAllForUserEntity(PublicKeyCredentialUserEntity $publicKeyCredentialUserEntity): array
	{
		$sources = [];
		foreach ($this->read() as $data)
		{
			$source = PublicKeyCredentialSource::createFromArray($data);
			if ($source->getUserHandle() === $publicKeyCredentialUserEntity->getId())
			{
				$sources[] = $source;
			}
		}
		return $sources;
	}

	/**
	 * @param PublicKeyCredentialSource $publicKeyCredentialSource
	 */
	public function saveCredentialSource(PublicKeyCredentialSource $publicKeyCredentialSource): void
	{
		$data = $this->read();
		$data[base64_encode($publicKeyCredentialSource->getPublicKeyCredentialId())] = $publicKeyCredentialSource;
		$this->write($data);
	}

	/**
	 * @return array
	 */
	private function read(): array
	{
		if (file_exists($this->path))
		{
			return json_decode(file_get_contents($this->path), true);
		}
		return [];
	}

	/**
	 * @param array $data
	 */
	private function write(array $data): void
	{
		file_put_contents($this->path, json_encode($data), LOCK_EX);
	}
}