<?php /** @noinspection PhpMissingReturnTypeInspection */

namespace DBTech\Security\XF\Admin\Controller;

use DBTech\Security\Helper;

class Login extends XFCP_Login
{
	/**
	 * @return \XF\Mvc\Reply\AbstractReply
	 * @throws \XF\Mvc\Reply\Exception
	 */
	public function actionLogin()
	{
		$this->assertPostOnly();

		if (\XF::visitor()->user_id)
		{
			return $this->redirect($this->getDynamicRedirectIfNot('login'));
		}

		if ($this->options()->dbtechSecurityLoginCaptcha['admin']
			&& !$this->captchaIsValid(true)
		) {
			return $this->error(\XF::phrase('did_not_complete_the_captcha_verification_properly'));
		}

		return parent::actionLogin();
	}


	/**
	 * @param \XF\Entity\User $user
	 *
	 * @throws \Exception
	 */
	protected function completeLogin(\XF\Entity\User $user)
	{
		parent::completeLogin($user);

		$watcherRepo = Helper::repository(\DBTech\Security\Repository\Watcher::class);
		$handler = $watcherRepo->getHandler('compromisedstafflogon', false);

		if ($handler)
		{
			$handler->trigger([
				'ipaddresses' => $this->request->getAllIps()
			], $user);
		}
	}
}