<?php /** @noinspection PhpMissingReturnTypeInspection */

namespace DBTech\Security\XF\ControllerPlugin;

use DBTech\Security\Helper;

/**
 * @extends \XF\ControllerPlugin\Login
 */
class Login extends XFCP_Login
{
	/**
	 * @param $userId
	 *
	 * @return mixed|null
	 */
	public function setDeviceTrusted($userId)
	{
		$key = parent::setDeviceTrusted($userId);

		if (!$this->request->filter('trust_permanent', 'bool'))
		{
			return $key;
		}

		$trustedUntil = \XF::$time + 86400 * 365;
		$this->app->response()->setCookie('tfa_trust', $key, $trustedUntil);

		/** @var \XF\Repository\UserTfaTrusted $tfaTrustRepo */
		$tfaTrustRepo = Helper::repository(\XF\Repository\UserTfaTrusted::class);
		$record = $tfaTrustRepo->getTfaTrustRecord($userId, $key);

		$record->fastUpdate('trusted_until', $trustedUntil);

		return $key;
	}

	/**
	 * @param \XF\Entity\User $user
	 * @param $remember
	 *
	 * @throws \XF\PrintableException
	 * @throws \Exception
	 */
	public function completeLogin(\XF\Entity\User $user, $remember)
	{
		parent::completeLogin($user, $remember);

		$watcherRepo = Helper::repository(\DBTech\Security\Repository\Watcher::class);
		$handler = $watcherRepo->getHandler('compromisedlogon', false);

		if ($handler)
		{
			$handler->trigger([
				'ipaddresses' => [$this->request->getIp()]
			], $user);
		}
	}
}