<?php /** @noinspection PhpMissingReturnTypeInspection */

namespace DBTech\Security\XF\Job;

use DBTech\Security\Helper;

/**
 * @extends \XF\Job\UserAction
 */
class UserAction extends XFCP_UserAction
{
	/**
	 * @param \XF\Entity\User $user
	 */
	protected function applyInternalUserChange(\XF\Entity\User $user)
	{
		parent::applyInternalUserChange($user);

		if ($this->getActionValue('dbtech_security_set_user_lock'))
		{
			/** @var \XF\Entity\UserOption $option */
			$option = $user->getRelationOrDefault('Option', false);

			if (!$option->use_tfa
				|| !$this->getActionValue('dbtech_security_set_user_lock_exclude_tfa')
			) {
				$option->dbtech_security_is_user_locked = true;
				$option->dbtech_security_is_admin_locked = false;

				$user->addCascadedSave($option);

				/** @var \DBTech\Security\Entity\AccountLock $lock */
				$lock = $user->getRelationOrDefault('DBTechSecurityAccountLock', false);

				$lock->bulkSet([
					'user_id'  => $user->user_id,
					'is_staff' => $user->is_staff,
					'action'   => 'user',
					'hash'     => \md5(\XF::generateRandomString(32)),
				]);

				$user->addCascadedSave($lock);
			}
		}

		if ($this->getActionValue('dbtech_security_force_password_change')
			&& !$this->getActionValue('dbtech_security_reset_password')
		) {
			/** @var \XF\Entity\UserOption $option */
			$option = $user->getRelationOrDefault('Option', false);

			if (!$option->use_tfa
				|| !$this->getActionValue('dbtech_security_force_password_change_exclude_tfa')
			) {
				$user->dbtech_security_forcenewpass = true;
			}
		}

		/** @var \XF\Entity\UserAuth $userAuth */
		$userAuth = $user->getRelationOrDefault('Auth', false);
		if ($this->getActionValue('dbtech_security_reset_password')
			&& $user->email
		) {
			/** @var \XF\Entity\UserOption $option */
			$option = $user->getRelationOrDefault('Option', false);

			if (!$option->use_tfa
				|| !$this->getActionValue('dbtech_security_reset_password_exclude_tfa')
			) {
				$userAuth->resetPassword();
				$user->addCascadedSave($userAuth);

				/** @var \XF\Service\User\PasswordReset $passwordReset */
				$passwordReset = Helper::service(\XF\Service\User\PasswordReset::class, $user);
				$passwordReset->setAdminReset(true);
				$passwordReset->triggerConfirmation();
			}
		}
	}
}