<?php /** @noinspection PhpMissingReturnTypeInspection */

namespace DBTech\Security\XF\Repository;

use DBTech\Security\Helper;

class Option extends XFCP_Option
{
	/**
	 * @param array $values
	 *
	 * @return \XF\Mvc\Entity\ArrayCollection
	 * @throws \XF\PrintableException
	 * @throws \Exception
	 */
	public function updateOptions(array $values)
	{
		$watcherRepo = Helper::repository(\DBTech\Security\Repository\Watcher::class);
		$handler = $watcherRepo->getHandler('option', false);

		if (!$handler)
		{
			return parent::updateOptions($values);
		}

		$oldValues = [];

		/** @var \XF\Entity\Option[] $options */
		$options = Helper::findByIds(\XF\Entity\Option::class, array_keys($values));

		foreach ($options as $option)
		{
			$oldValue = $option->option_value;
			if (is_array($oldValue))
			{
				$oldValue = json_encode($oldValue);
			}

			$oldValues[$option->option_id] = $oldValue;
		}

		$retval = parent::updateOptions($values);
		$this->em->clearEntityCache('XF:Option');

		/** @var \XF\Entity\Option[] $options */
		$options = Helper::findByIds(\XF\Entity\Option::class, array_keys($values));

		foreach ($options AS $option)
		{
			$oldValue = $oldValues[$option->option_id];
			$newValue = $option->option_value;

			if (is_array($newValue))
			{
				$newValue = json_encode($newValue);
			}

			if (strval($oldValue) === strval($newValue))
			{
				continue;
			}

			$handler->trigger([
				'ipaddress' 	=> $this->app()->request()->getIp(),
				'script' 		=> 'options',
				'action' 		=> 'edited',
				'id' 			=> 0,
				'title' 		=> $option->title,
				'field' 		=> $option->option_id,
				'old' 			=> strval($oldValue),
				'new' 			=> strval($newValue),
				'differences'  	=> $oldValue . ' -> ' . $newValue
			], \XF::visitor());
		}

		return $retval;
	}
}