<?php /** @noinspection PhpMissingReturnTypeInspection */

namespace DBTech\Security\XF\Session;

use DBTech\Security\Helper;

class Session extends XFCP_Session
{
	/**
	 * @param $ownerIp
	 * @param null $sessionId
	 *
	 * @return \XF\Session\Session
	 */
	public function start($ownerIp, $sessionId = null)
	{
		if ($sessionId)
		{
			$sessionRepo = Helper::repository(\DBTech\Security\Repository\Session::class);

			$_sessionId = $sessionRepo->getSessionCookie();
			if ($_sessionId)
			{
				// Check to see if we can get our session
				$session = Helper::find(\DBTech\Security\Entity\Session::class, $_sessionId);
				if (!$session || $session->isExpired())
				{
					// Delete session cookie
					$sessionRepo->deleteSessionCookie();

					// Create a new session for us
					return parent::start($ownerIp);
				}
			}
		}

		return parent::start($ownerIp, $sessionId);
	}

	/**
	 * @param \XF\Entity\User $user
	 *
	 * @return \XF\Session\Session
	 * @throws \XF\Db\Exception
	 */
	public function changeUser(\XF\Entity\User $user)
	{
		parent::changeUser($user);

		$sessionRepo = Helper::repository(\DBTech\Security\Repository\Session::class);

		$session = null;
		$sessionId = $sessionRepo->getSessionCookie();
		if ($sessionId)
		{
			$session = Helper::find(\DBTech\Security\Entity\Session::class, $sessionId);
		}

		if ($session && !$session->isExpired())
		{
			$sessionRepo->updateSessionLastActive($session, $user);
		}
		else
		{
			$sessionRepo->createSession($user);
		}

		return $this;
	}

	/**
	 * @return bool
	 * @throws \XF\Db\Exception
	 */
	public function save()
	{
		$sessionRepo = Helper::repository(\DBTech\Security\Repository\Session::class);

		$session = null;
		$sessionId = $sessionRepo->getSessionCookie();
		if ($sessionId)
		{
			$session = Helper::find(\DBTech\Security\Entity\Session::class, $sessionId);
		}

		if ($session && !$session->isExpired())
		{
			$sessionRepo->updateSessionLastActive($session);
		}

		return parent::save();
	}
}